package Logoved::Stream::LogParser;
use Moo;
#use strict;
#use warnings;
use File::Find;
use File::Basename;
use Logoved::Stream::In;
use Logoved::Stream::Pump;

has 'listeners_factory' => (is => 'rw');
has 'parser_class' => (is => 'rw');

has 'filepath_is_logfile' => (is => 'rw');
has 'dir_ignore_empty_files' => (is => 'rw', default => 1);
has 'dir_ignore_filepatterns' => (
    is => 'ro',
    isa => sub { die "internal error: dir_ignore_filepatterns: should be a ref to an array of patterns!" if $_[0] and not ref($_[0]) eq 'ARRAY'; },
    default => sub {[qr'^index\.html?$']},
    );
has 'ignore_filepatterns' => (
    is => 'ro',
    isa => sub { die "internal error: ignore_filepatterns: should be a ref to an array of patterns!" if $_[0] and not ref($_[0]) eq 'ARRAY'; },
    );

sub parse_file {
    my ($self, $input) =@_;
    my $parser_class=$self->parser_class;
    die "parser_class is not set" unless $parser_class;
    die "is a directory: $input" if -d $input;
    my $name=basename($input);
    my $ignore_filepatterns=$self->ignore_filepatterns;
    if ($ignore_filepatterns) {
	foreach my $pattern (@$ignore_filepatterns) {
	    if ($name =~ $pattern) {
		#warn 'info: ignored ',$name, ' by filepattern ',$pattern,"\n";
		return;
	    };
	}
    }
    my $listeners_factory=$self->listeners_factory;
    my $instream=Logoved::Stream::In->create_from_file($input);
    $listeners_factory->init_new_stream($instream) if $listeners_factory;
    my $outstream=$parser_class->new(
	source => $instream,
	listeners_factory => $listeners_factory,
	);
    Logoved::Stream::Pump->new(
	in => $instream,
	out => $outstream,
	);
    $listeners_factory->done_new_stream($instream) if $listeners_factory;
    $instream->commit;
}

sub parse_dir {
    my ($self, $input) =@_;
    my $filepath_is_logfile=$self->filepath_is_logfile;
    my $dir_ignore_empty_files=$self->dir_ignore_empty_files;
    my $dir_ignore_filepatterns=$self->dir_ignore_filepatterns;
    my @files;
    find(sub {
	return if -d $_;
	return if $filepath_is_logfile and not &$filepath_is_logfile($File::Find::name);
	if ($dir_ignore_filepatterns) {
	    foreach my $pattern (@$dir_ignore_filepatterns) {
		if ($_ =~ $pattern) {
		    warn 'LogParser: parse_dir: ignored file ',$_,"\n";# ' by filepattern ',$pattern,"\n";
		    return;
		};
	    }
	}
	if ($dir_ignore_empty_files and ! -s $_) {
	    warn "LogParser: parse_dir: file $File::Find::name is empty. Ignored.\n"
		if $dir_ignore_empty_files<2;
	    return;
	}
	push @files, $File::Find::name;
	 }, $input);
    map {$self->parse_file($_)} @files;
}

sub parse_arg {
    my ($self, $input) =@_;
    if (-d $input) {
	$self->parse_dir($input);
    } elsif (-e $input) {
	$self->parse_file($input);
    } else {
	die "LogParser: input $input does not exist.\n";
    }
}

1;
__END__
